/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
localScripts = {
    aria: { name: "aria", url: "js/aria-web-telemetry-2.7.1.min" },
    jquery: { name: "jquery", url: "js/jquery-3.3.1.min" },
    jqueryui: { name: "jqueryui", url: "js/jquery-ui.min", require: { deps: ['jquery'] } },
    msajax: { name: "msajax", url: "js/MicrosoftAjax", require: { exports: 'Sys' } },
    mscorlib: { name: "mscorlib", url: "js/mscorlib", require: { exports: 'ss' } }
};

localRootScriptNames = ['framework', 'localization']
 
var app = {
    // Launch uri used in Win platform
    winLaunchUri: null,
    shimDeviceCategory: null,
    server: null, 
    firstRun: true,
    applicationId: "$APPLICATIONID$",

    // Application Constructor
    initialize: function() {
        this.bindEvents();
    },
	
    // Bind Event Listeners
    //
    // Bind any events that are required on startup. Common events are:
    // 'load', 'deviceready', 'offline', and 'online'.
    bindEvents: function() {
        if (window.WinJS) {
            // Cordova Windows 4.3.2 has a bug where not all information is passed from Win activated event. So we have to handle native event ourselves
            // TODO: remove this and update protocol handler plugin after updating Cordova Windows platform
            WinJS.Application.addEventListener("activated", this.onWinAppActivated, false);
        }
        document.addEventListener('deviceready', this.onDeviceReady, false);
    },

    // deviceready Event Handler
    //
    // The scope of 'this' is the event. In order to call the 'receivedEvent'
    // function, we must explicitly call 'app.receivedEvent(...);'
    onDeviceReady: function() {
        if (history.length > 1) {
            // Reseting the webview history in case some entries has been left over.
            history.go(-(history.length - 1));
        }

        if (device.platform && device.platform.toLowerCase() == "android") {
            if (device.memory && device.memory < 1.5 * 1024 * 1024 * 1024) { // Defining less than 1.5 GB of memory to be older device.
                // When run in the current version on the Chrome backed webview on older devices we have to not use any css transition/translate functions due to lag of proper hardware acceleration.
                document.documentElement.className = "no-animation";
            }
        }

        app.loadResources(
            function () {
                // Mapped on AppOptions class
                var appOptions = { 
                    "applicationId": app.applicationId,
                    "firstRun": app.firstRun,
                    "attributes":{},
                };

                if (DN && DN.SignInTelemetryLogger && DN.SignInTelemetryLogger.enableTelemetry) {
                    DN.SignInTelemetryLogger.enableTelemetry()
                }

                // Shim should control which device category is selected. On IOS it is possible that shim will load with one user agent and then webview will load with different
                if(app.shimDeviceCategory != null){
                    appOptions.attributes["DeviceCategory"] = app.shimDeviceCategory;
                }
                if (Microsoft.Dynamics.NAV) {
                    Microsoft.Dynamics.NAV.App.initialize(appOptions);
                } else {
                    // In order to support NAV 2015 (8.0) we need to invoke App object using different namespace (Nav)
                    Microsoft.Dynamics.Nav.App.initialize(appOptions);
                }

                cordova.fireDocumentEvent('appready', null, true);
            });
    },

    // Activated event only triggers on windows. We use it to get launch uri
    onWinAppActivated: function (args) {
        if (args.detail.uri) {
            if (args.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.protocol) {
                app.winLaunchUri = args.detail.uri.absoluteUri;
            }
        }
    },
    
    localAppDefinition: {
        version: 2000,
        resources: {
            scripts: [
                localScripts.jquery,
                localScripts.jqueryui,
                localScripts.msajax,
                localScripts.mscorlib,
                { name: "logicalclientDll", url: "js/logicalclientDll", require: { exports: 'LCDLL' } },
                { name: "logicalclient", url: "js/logicalclient", require: { exports: 'logicalclient', deps: ['logicalclientDll'] } },
                { name: "frameworkBaseDll", url: "js/frameworkBaseDll", require: { exports: 'DNFBDLL' } },
                { name: "frameworkBase", url: "js/frameworkBase", require: { exports: 'DNFB', deps: ['frameworkBaseDll'] } },
                { name: "framework", url: "js/Framework", require: { exports: 'DN', deps: ['jquery', 'jqueryui', 'msajax', 'mscorlib', 'frameworkBase', 'logicalclient'] } },
                { name: "localization", url: "js/ShimLocalizedStrings", require: { deps: ['framework'] } }
            ],
            rootScriptNames: localRootScriptNames,
            stylesheets: [
                "Resources/{cssName}.css?v=" + Date.now()
            ]
        }
    },

    loadResources: function (done) {
        var bootstrapInfo = window.sessionStorage.getItem("__bootstrapInfo");
        var appDefinition;
        if (bootstrapInfo === undefined || bootstrapInfo === null) {
            appDefinition = app.localAppDefinition;
        } else {
            bootstrapInfo = JSON.parse(bootstrapInfo);
            window["__bootstrapInfo"] = bootstrapInfo;
            window.sessionStorage.removeItem("__bootstrapInfo");
            app.firstRun = false;
            appDefinition = bootstrapInfo.appDefinition;
            app.shimDeviceCategory = bootstrapInfo.deviceCategory;
            app.server = bootstrapInfo.server;
            if (appDefinition.configuration && appDefinition.configuration.ApplicationId) {
                app.applicationId = appDefinition.configuration.ApplicationId;
            }
        }

        app.loadScripts(
            appDefinition.version,
            appDefinition.resources.scripts,
            appDefinition.resources.rootScriptNames,
            function () {
                if (DN && DN.AppDefinition) {
                    DN.AppDefinition.current = appDefinition;
                    if (DN.ClientConfiguration) {
                        DN.ClientConfiguration.initialize(appDefinition.configuration);
                    }
                }
                if (appDefinition.resources.stylesheets) {
                    app.loadStylesheets(appDefinition.resources.stylesheets, appDefinition.resources.stylesheetAreas, done);
                } else {
                    done();
                }
            });
    },

    loadScripts: function (version, scripts, rootScriptNames, done) {
        if (version >= 2000) {
            app.loadScriptsViaRequire(scripts, rootScriptNames, done);
            return;
        }
        var index = 0;
        var loadAScript = function () {
            if (index === scripts.length) {
                done();
            } else {
                app.loadScript(scripts[index++].url, function () { loadAScript(); });
            }
        };
        loadAScript();
    },

    loadScript: function (src, done) {
        var script = document.createElement("script");
        script.type = "text/javascript";
        script.onload = function () {
            done();
        };
        script.onerror = function () {
            window.sessionStorage.setItem("__bootstrap_failed", true);
            document.location.reload();
        };
        try {
            script.src = src;
            document.getElementsByTagName("head")[0].appendChild(script);
        } catch (e) {
            window.sessionStorage.setItem("__bootstrap_failed", true);
            document.location.reload();
        }
    },

    loadScriptsViaRequire: function (scripts, rootScriptNames, done) {
        requirejs.config(app.buildRequireConfig(scripts));
        if (rootScriptNames === undefined || rootScriptNames === null) {
            rootScriptNames = localRootScriptNames;
        }
        require(rootScriptNames,
            function () {
                done();
            },
            function () {
                window.sessionStorage.setItem("__bootstrap_failed", true);
                document.location.reload();
            });
    },

    // Don't use local copy of script if different from that required by the web client
    detectLocalScriptVersionMismatch: function (script, regExp) {
        if (localScripts[script.name]) {
            var match = localScripts[script.name].url.match(regExp);

            if (match) {
                var firstMatch = match[0];
                if(!script.url.includes(firstMatch)) {
                    script.useLocal = false;
                }
            }
        }
        return script;
    },

    // Correct malformed URLs in the bootstrap data if from the web client's origin (i.e. not from a CDN or no origin provided before login)
    correctWebClientHostScriptURL: function(script, regExp) {
        if (app.server) {
            var serverURL = new URL(app.server);

            if (script.url.includes(serverURL.origin)) {
                var pathMatch = script.url.match(regExp);

                // Give up if can't find the path regular expression
                if (pathMatch) {
                    var firstMatch = pathMatch[0];

                    // Some server URLs are missing a trailing slash. Two successive slashes in path navigates to same folder so no error. 
                    var correctedBase = new URL(serverURL.pathname, serverURL.origin) + "/js/";
                    var correctedURL = new URL(firstMatch, correctedBase);
                    script.url = correctedURL.href;
                }
            }
        }
        return script;
    },

    buildRequireConfig: function (scripts) {
        var paths = {};
        var shim = {};
        for (var i = 0, l = scripts.length; i < l; i++) {
            var script = scripts[i];

            if (script.name == "jquery") {
                var jqueryRegExp = new RegExp("jquery-.*\.min");

                script = app.detectLocalScriptVersionMismatch(script, jqueryRegExp);
                script = app.correctWebClientHostScriptURL(script, jqueryRegExp);
            }

            if (script.useLocal && localScripts[script.name]) {
                script = localScripts[script.name];
                scripts[i] = script;
            }
            paths[script.name] = script.url;
            if (script.require) {
                shim[script.name] = script.require;
            }
        }

        // By default require.js has a timeout of 7 seconds.
        // We want to disable it because our scripts can take some time to download (e.g. in debug mode framework.s is more than 4mb)
        // In order to disable timeout set 'waitSeconds' to 0
        return { 'paths': paths, 'shim': shim, 'waitSeconds': 0 };
    },

    // This logic is needed when we do not bootstrap
    chooseStylesheet: function () {
        var cssName = "tablet";
        if (DN && DN.DeviceCategoryDetector) {
            var deviceCategory = DN.DeviceCategoryDetector.detectAsString();
            // the cssName for desktop and tablet is tablet.css
            cssName = deviceCategory === 'desktop' ? 'tablet' : deviceCategory;
        }
        return cssName;
    },

    loadStylesheets: function (stylesheets, stylesheetAreas, done) {
        var cssName = app.chooseStylesheet();

        stylesheets.push("Resources/projectmadeira.css");
        
        var loadedScripts = 0;
        var cssLoadSuccessCallback = function(){
            // check if all stylesheets have completed loading.
            if (++loadedScripts === stylesheets.length) {
                done();
            }
        }

        for (var i = 0, l = stylesheets.length; i < l; i++) {
            var link = document.createElement("link");
            link.href = stylesheets[i].replace("{cssName}", cssName);
            link.type = "text/css";

            // Check if stylesheet needs to be loaded on-demand.
            if(stylesheetAreas && stylesheetAreas[i] != 0){
                link.setAttribute("data-cssarea", stylesheetAreas[i]);
                cssLoadSuccessCallback();
            }else{
                link.rel = "stylesheet";
                link.onload = function(e) {
                    cssLoadSuccessCallback();
                };
    
                link.onerror = function() {
                    window.sessionStorage.setItem("__bootstrap_failed", true);
                    document.location.reload();
                };
            }

            document.getElementsByTagName("head")[0].appendChild(link);
        }
    }
};

app.initialize();